
% lab06_ex_fft_usage.m - 
clear all; close all;

% Signal
if(0) % Raport: "Spectrum and spectral density estimation by the DFT ..."
      % G. Heinzel, A. Rudiger and R. Schilling, Max-Planck Institut 2001
   N=13*256; fs=10000; xlsb=0.001; dt=1/fs; t=dt*(0:N-1);
   x = 1*sin(2*pi*1234*t) + sqrt(2)*sin(2*pi*2500.2157*t);
end
if(1) % Our book
   N=5000; fs=10000; xlsb=0.001; % N=5000=(2^3)*(5^4)=8*625 = fast alg.
   dt=1/fs; t=dt*(0:N-1);
   x = 1*sin(2*pi*1000*t) + sqrt(2)*sin(2*pi*3501.1234*t);
end

% Noise - alternative: SNR=80(?); x = awgn(x,SNR,'measured');
x = floor( x/xlsb + 0.5 )*xlsb;  noise_level = xlsb/sqrt(6*fs),

% Windowing
w1 = rectwin(N)'; w2 = hann(N)'; w3 = kaiser(N,12)'; w4 = flattopwin(N)';
w  = w4;
x  = x .* w;

% Spectral estimation of mixtures of sines
X  = fft(x); Xa = abs(X);       % FFT without scaling, no division (by sqrt(N),N)
XN = X/N;                       % FFT with typical division by N (rectangular wind)
Xas = 2*Xa/sum(w);              % Ampl spectrum  (AS) - correct for any window
Xps = Xas.^2/2;                 % Power spectrum (PS) - correct for any window
Xasd = 2*Xa/sqrt(fs*sum(w.^2)); % Amplitude spectral density (ASD) - as above
Xpsd = Xasd.^2/2;               % Power spectral density (PSD) - as above

%  PS = AS^2/2; ASD=AS/sqrt(ENBW); PSD = ASD^2/2 = PS/ENBW = AS^2/ENBW; 
S1 = sum(w); S2 = sum(w.^2); ENBW = fs*S2/S1^2, NENBW = N*S2/S1^2,
err_asd = max(abs(Xasd-Xas/sqrt(ENBW))),
err_psd = max((abs(Xpsd)-Xas.^2/ENBW)), pause

    % Figures
    k = 1:N/2+1;                    % indexes of non-negative frequencies
    f0 = fs/N, f = f0 * (k-1);      % none-negative frequencies (1st spectrum half) 
    figure;
    subplot(221); stem(f(k),Xas(k));  grid; axis tight; xlabel('f (Hz)'); ylabel('[V]'); title('Amplitude spectrum');
    subplot(222); stem(f(k),Xasd(k)); grid; axis tight; xlabel('f (Hz)'); ylabel('[V/\surdHz]'); title('Amplitude spectral density');
    subplot(223); stem(f(k),Xps(k));  grid; axis tight; xlabel('f (Hz)'); ylabel('[V^2]'); title('Power spectrum');
    subplot(224); stem(f(k),Xpsd(k)); grid; axis tight; xlabel('f (Hz)'); ylabel('[V^2/Hz]'); title('Power spectral density');
    pause
    figure;
    subplot(221); semilogy(f(k),Xas(k));  grid; axis tight; xlabel('f (Hz)'); ylabel('[V]'); title('Amplitude spectrum');
    subplot(222); semilogy(f(k),Xasd(k)); grid; axis tight; xlabel('f (Hz)'); ylabel('[V/\surdHz]'); title('Amplitude spectral density');
    subplot(223); semilogy(f(k),Xps(k));  grid; axis tight; xlabel('f (Hz)'); ylabel('[V^2]'); title('Power spectrum');
    subplot(224); semilogy(f(k),Xpsd(k)); grid; axis tight; xlabel('f (Hz)'); ylabel('[V^2/Hz]'); title('Power spectral density');
    pause
    figure;
    subplot(221); plot(f(k),20*log10(Xas(k)));  grid; axis tight; xlabel('f (Hz)'); ylabel('[V]'); title('Amplitude spectrum');
    subplot(222); plot(f(k),20*log10(Xasd(k))); grid; axis tight; xlabel('f (Hz)'); ylabel('[V/\surdHz]'); title('Amplitude spectral density');
    subplot(223); plot(f(k),10*log10(Xps(k)));  grid; axis tight; xlabel('f (Hz)'); ylabel('[V^2]'); title('Power spectrum');
    subplot(224); plot(f(k),10*log10(Xpsd(k))); grid; axis tight; xlabel('f (Hz)'); ylabel('[V^2/Hz]'); title('Power spectral density');
    pause

% Spectral estimation of noisy signals
% Taking a longer signal fragment and cutting it into shorter fragments
% Then calculation of many short spectra and the mean spectrum
% [X,f] = pwelch(x,Nwin,Nover,Nfft,fs);   % in Matlab and Octave

M = 100; NM=N*M; n = 0 : NM-1; t=dt*(0:NM-1);            % CHOOSE:
x = 1*sin(2*pi*1000*t) + sqrt(2)*sin(2*pi*3501.1234*t);  % two sines
%x = 1*sin(2*pi*(1000*t + 0.5*50*t.^2));                 % LFM signal: +50 Hz/s
x = floor( x/xlsb + 0.5 )*xlsb;  % noise addition
X = reshape(x,N,M);              % put signal fragments into M columns of matrix X
W = repmat( w', 1, M);           % put the same window w into M columns of matrix W 
Xa = abs( fft( X.*W ) );         % perform FFT of each column of X.*W, then abs()
Xasd2 = 2*Xa/sqrt(fs*sum(w.^2)); % many Amplitude Spectral Densities (ASD)
Xpsd2 = Xasd2.^2/2;              % many Power Spectral Densities (PSD)
% Calculation of a mean spectrum: cols (spectra) --> rows, mean of rows
Xasd2 = sum(Xasd2')/M;           % mean ASD         
Xpsd2 = sum(Xpsd2')/M;           % mean PSD

    figure
    subplot(121); semilogy(f(k),Xasd(k),'g--',f(k),Xasd2(k),'b.-'); grid; axis tight; xlabel('f (Hz)'); ylabel('[V/\surdHz]]'); title('Amplitude SD');
    subplot(122); semilogy(f(k),Xpsd(k),'g--',f(k),Xpsd2(k),'b.-'); grid; axis tight; xlabel('f (Hz)'); ylabel('[V^2/Hz]]'); title('Power SD');
    pause

% Spectrogram - Short-Time Fourier Transform (STFT)
% sequence of many amplitude spectra for consecutive signal fragments
% [X,f,t] = spectrogram(x,Nwin,Nover,Nfft,fs);   % STFT in Matlab (new)
% [X,f,t] = specgram(x,Nfft,fs,Nwin,Nover);      % STFT in Octave and Matlab(old)
 
Xas = 2*Xa(1:N/2+1,1:M)/sum(w);  % absolute values, positive frequencies, scaling 
t = dt*(N/2 : N : N/2+(M-1)*N);

    figure
    mesh(t,f,20*log10(Xas)); view(-40,70); axis tight; % AS matrix as a 3D mesh
    cb=colorbar('location','EastOutside'); set( get(cb,'Ylabel'),'String','V (dB)');
    xlabel('time (s)'); ylabel('frequency (Hz)'); pause
    figure
    imagesc(t,f,20*log10(Xas));     % AS matrix as an image
    cb=colorbar('location','EastOutside'); set( get(cb,'Ylabel'),'String','V (dB)');
    xlabel('time (s)'); ylabel('frequency (Hz)'); pause

% For comparison - one mean spectrum of many short signal fragments
Xas = 2*Xa/sum(w);                 % many amplitude spectra (AS)
Xasm = sum(Xas')/M;                % mean AS, cols --> rows, sum of rows  

    figure
    subplot(111); semilogy(f(k),Xasm(k),'b'); grid; axis tight; xlabel('f (Hz)'); ylabel('(V)'); title('Mean amplitude spectrum');
    pause

% For comparison - one spectrum of very long signal fragment
w = hann(NM)';                     % very long window 
Xas1 = 2*abs(fft(x.*w))/sum(w);    % one AS of the whole long signal
k = 1:1:NM/2+1; f=fs/NM*(0:NM-1);  % frequencies of AS coefficients

    figure
    subplot(111); semilogy(f(k),Xas1(k),'b'); grid; axis tight; xlabel('f (Hz)'); ylabel('(V)'); title('One long amplitude spectrum');
