
% lab06_ex_ipdft.m  Interpolated DFT/FFT
clear all; close all;

% Test signal
N=256; fs=100;             % number of samples, sampling ratio [Hz]
Ax=6; dx=0.5; fx=4; px=3;  % signal amplitude, damping, frequency, phase
dt=1/(fs); t=(0:N-1)*dt;   % sampling time
x = Ax*exp(-dx*t).*cos(2*pi*fx*t+px); plot(t,x); pause % signal generation

% Interpolated DFT for maksimum absolute DFT peak 
Xw = fft(x); plot(abs(Xw)), pause          % computation of DFT/FFT
[Xabs, ind] = max(abs(Xw(1:round(N/2))));  % find maximum and its index
km1 = ind-1; k=ind; kp1 = ind+1;           % three DFT samples around max
dw = 2*pi/N; 	           % DFT frequency step
wkm1= (km1-1)*dw;	       % frequency of DFT bin with index k-1
wk  = (k  -1)*dw;	       % frequency of DFT bin with index k
wkp1= (kp1-1)*dw;	       % frequency of DFT bin with index k+1
r = ( -exp(-j*wk)+exp(-j*wkm1) )/( -exp(-j*wkp1)+exp(-j*wk) ); % eq.(6.43)
R = ( Xw(km1)-Xw(k) )/( Xw(k)-Xw(kp1) );                       % eq.(6.42)
lambda = exp(j*wk)*(r-R)/( r*exp(-j*2*pi/N)-R*exp(j*2*pi/N) ); % eq.(6.44)
we = imag(log(lambda));    % estimated radial pulsation
de = -real(log(lambda));   % estimated damping
fe = we*fs/(2*pi);         % radial pulsation --> frequency
de = de*fs;                % normalized damping (de/fs) --> damping (de)

if round(1e6*R)==-1e6      % COHERENT SAMPLING, dx=0
   Ae = 2*abs(Xw(k))/N;    % estimated amplitude
   pe = angle(Xw(k));      % estimated phase
else                       % NON-COHERENT SAMPLING
   c = (1-lambda^N)/(1-lambda*exp(-j*wk));                    % eq.(6.47)
   c = 2*Xw(k)/c;                                             % eq.(6.47)
   Ae = abs(c);            % estimated amplitude              % eq.(6.47)
   pe = angle(c);          % estimated phase                  % eq.(6.47) 
end

result = [ Ae, de, fe, pe ],                    % display results
errors = [ Ae-Ax, de-dx, fe-fx, pe-px ], pause  % show errors

