% lab06_ex_psd.m  Blackman-Tukey PSD compared with Welch PSD

  clear all; close all; subplot(111);

% Parameters  
  N = 256; fs = 1000;                   % number of samples, sampling ratio 
  M = N/4;                              % number of computed AutoCorr coeffs
  df = fs/N, f=0:df:(N-1)*df;           % PSD spectrum frequencies
  dt=1/fs; t=0:dt:(N-1)*dt;             % time stamps for signal
  tR=-(N-1)*dt:dt:(N-1)*dt; k=1:N/2+1;  % time stamps for auto-correlation

% Generation of analyzed signal - sinusoid 100 Hz in noise
  x = sin(2*pi*25.5*t) + 0.1*randn(1,N);      % signal + Gaussian noise
  figure; plot(t,x); grid; axis tight; title('x(n)'); xlabel('time [s]'); pause
  
% Fast computation of the first M coeffs of the auto-correlation using FFT
  xz = [x zeros(1,M-1)];                      % appending M-1 zeros
  X = fft(xz); X = X.*conj(X); R = ifft(X);   % FFT, |X(k)|^2, IFFT
  R = real(R(1:M))/N;                         % coeffs of interest, scaling
  
% PDS estimation - Blackman-Tukey method = Fourier transform of signal autocorr
  w = hanning(2*M-1); w=w(M:2*M-1); w=w';     % choose window
  Rw = R .* w;                                % multiply auto-corr with window
  s = [ Rw(1:M) zeros(1,N-2*M+1) Rw(M:-1:2)]; % input to FFT is symmetrical
  P1  = real(fft(s))/fs;                      % the FFT result, real-value vector
  figure; subplot(211); plot(f(k),P1(k),'b'); grid;
  title('Blackman-Tukey estimation of PSD'); xlabel('f [Hz]'); ylabel('V^2 / Hz');
  
% PSD estimation - Welch method = averaging periodograms of signal fragments
  Nfft = N; Nwind = 2*M; Noverlap = Nwind/2;  % lengths: FFT, window, overlap
  Nshift = Nwind-Noverlap;                    % window shift
  M = floor((N-Nwind)/Nshift)+1;              % number of signal fragments
  w = hanning(Nwind)';                        % window choice
  P2 = zeros(1,Nfft);                         % initialization
  for m=1:M                                   % number of the signal fragment
     n = 1+(m-1)*Nshift : Nwind+(m-1)*Nshift; % which samples?
     X = fft( x(n) .* w, Nfft);               % FFT of windowed fragment
     P2 = P2 + abs(X).^2;                     % accumulate
  end                                         % end of loop
  P2 = P2/(fs*M*sum(w.*w));                   % PSD normalization
  subplot(212); plot(f(k),P2(k),'b'); grid; title('Welch estimation of PSD'); xlabel('f [Hz]'); ylabel('V^2 / Hz'); pause
